@@ -68,10 +68,12 @@ impl<VM: VMBinding> GCWork<VM> for MSSweepChunks<VM> {
     }
 }
 
-use crate::scheduler::gc_work::SFTProcessEdges;
 pub struct MSGCWorkContext<VM: VMBinding>(std::marker::PhantomData<VM>);
+use crate::policy::gc_work::DEFAULT_TRACE;
+use crate::scheduler::gc_work::PlanProcessEdges;
+
 impl<VM: VMBinding> crate::scheduler::GCWorkContext for MSGCWorkContext<VM> {
     type VM = VM;
     type PlanType = MarkSweep<VM>;
-    type ProcessEdgesWorkType = SFTProcessEdges<Self::VM>;
+    type ProcessEdgesWorkType = PlanProcessEdges<Self::VM, MarkSweep<VM>, DEFAULT_TRACE>;
 }
