@@ -113,6 +113,10 @@ impl<VM: VMBinding> WorkBucket<VM> {
     pub fn add<W: GCWork<VM>>(&self, work: W) {
         self.add_with_priority(Self::DEFAULT_PRIORITY, Box::new(work));
     }
+    /// Add a boxed work packet to this bucket, with a default priority (1000)
+    pub fn add_boxed(&self, work: Box<dyn GCWork<VM>>) {
+        self.add_with_priority(Self::DEFAULT_PRIORITY, work);
+    }
     pub fn bulk_add_with_priority(&self, priority: usize, work_vec: Vec<Box<dyn GCWork<VM>>>) {
         {
             let mut queue = self.queue.write();
@@ -154,13 +158,17 @@ pub enum WorkBucketStage {
     Unconstrained,
     Prepare,
     Closure,
-    // TODO: We only support final reference at the moment. If we have references of multiple strengths,
-    // we may need more than one buckets for each reference strength.
-    RefClosure,
+    SoftRefClosure,
+    WeakRefClosure,
+    FinalRefClosure,
+    PhantomRefClosure,
     CalculateForwarding,
     SecondRoots,
     RefForwarding,
+    FinalizableForwarding,
     Compact,
     Release,
     Final,
 }
+
+pub const LAST_CLOSURE_BUCKET: WorkBucketStage = WorkBucketStage::PhantomRefClosure;
