@@ -48,7 +48,7 @@ pub struct GCWorker<VM: VMBinding> {
     /// The sending end of the channel to send message to the controller thread.
     pub sender: Sender<CoordinatorMessage<VM>>,
     /// The reference to the MMTk instance.
-    mmtk: &'static MMTK<VM>,
+    pub mmtk: &'static MMTK<VM>,
     /// True if this struct is the embedded GCWorker of the controller thread.
     /// False if this struct belongs to a standalone GCWorker thread.
     is_coordinator: bool,
@@ -140,6 +140,10 @@ impl<VM: VMBinding> GCWorker<VM> {
         work.do_work(self, self.mmtk);
     }
 
+    pub fn do_boxed_work(&'static mut self, mut work: Box<dyn GCWork<VM>>) {
+        work.do_work(self, self.mmtk);
+    }
+
     pub fn run(&mut self, tls: VMWorkerThread, mmtk: &'static MMTK<VM>) {
         self.tls = tls;
         self.copy = crate::plan::create_gc_worker_context(tls, mmtk);
