@@ -13,25 +13,28 @@ mod tests {
     #[test]
     fn test_side_metadata_address_to_meta_address() {
         let mut gspec = SideMetadataSpec {
+            name: "gspec",
             is_global: true,
             offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
             log_num_of_bits: 0,
-            log_min_obj_size: 0,
+            log_bytes_in_region: 0,
         };
         #[cfg(target_pointer_width = "64")]
         let mut lspec = SideMetadataSpec {
+            name: "lspec",
             is_global: false,
             offset: SideMetadataOffset::addr(LOCAL_SIDE_METADATA_BASE_ADDRESS),
             log_num_of_bits: 0,
-            log_min_obj_size: 0,
+            log_bytes_in_region: 0,
         };
 
         #[cfg(target_pointer_width = "32")]
         let mut lspec = SideMetadataSpec {
+            name: "lspec",
             is_global: false,
             offset: SideMetadataOffset::rel(0),
             log_num_of_bits: 0,
-            log_min_obj_size: 0,
+            log_bytes_in_region: 0,
         };
 
         assert_eq!(
@@ -61,8 +64,8 @@ mod tests {
             LOCAL_SIDE_METADATA_BASE_ADDRESS + 16usize
         );
 
-        gspec.log_min_obj_size = 2;
-        lspec.log_min_obj_size = 1;
+        gspec.log_bytes_in_region = 2;
+        lspec.log_bytes_in_region = 1;
 
         assert_eq!(
             address_to_meta_address(&gspec, unsafe { Address::from_usize(0) }),
@@ -125,10 +128,11 @@ mod tests {
     #[test]
     fn test_side_metadata_meta_byte_mask() {
         let mut spec = SideMetadataSpec {
+            name: "test_spec",
             is_global: true,
             offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
             log_num_of_bits: 0,
-            log_min_obj_size: 0,
+            log_bytes_in_region: 0,
         };
 
         assert_eq!(meta_byte_mask(&spec), 1);
@@ -144,10 +148,11 @@ mod tests {
     #[test]
     fn test_side_metadata_meta_byte_lshift() {
         let mut spec = SideMetadataSpec {
+            name: "test_spec",
             is_global: true,
             offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
             log_num_of_bits: 0,
-            log_min_obj_size: 0,
+            log_bytes_in_region: 0,
         };
 
         assert_eq!(
@@ -191,24 +196,27 @@ mod tests {
                     // We need to do this because of the static NO_METADATA
                     // sanity::reset();
                     let mut gspec = SideMetadataSpec {
+                        name: "gspec",
                         is_global: true,
                         offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
                         log_num_of_bits: 1,
-                        log_min_obj_size: 1,
+                        log_bytes_in_region: 1,
                     };
                     #[cfg(target_pointer_width = "64")]
                     let mut lspec = SideMetadataSpec {
+                        name: "lspec",
                         is_global: false,
                         offset: SideMetadataOffset::addr(LOCAL_SIDE_METADATA_BASE_ADDRESS),
                         log_num_of_bits: 1,
-                        log_min_obj_size: 1,
+                        log_bytes_in_region: 1,
                     };
                     #[cfg(target_pointer_width = "32")]
                     let mut lspec = SideMetadataSpec {
+                        name: "lspec",
                         is_global: false,
                         offset: SideMetadataOffset::rel(0),
                         log_num_of_bits: 1,
-                        log_min_obj_size: 1,
+                        log_bytes_in_region: 1,
                     };
 
                     let metadata = SideMetadataContext {
@@ -242,9 +250,9 @@ mod tests {
                         constants::BYTES_IN_PAGE,
                     );
 
-                    gspec.log_min_obj_size = 4;
+                    gspec.log_bytes_in_region = 4;
                     gspec.log_num_of_bits = 4;
-                    lspec.log_min_obj_size = 4;
+                    lspec.log_bytes_in_region = 4;
                     lspec.log_num_of_bits = 4;
 
                     metadata_sanity.reset();
@@ -305,17 +313,19 @@ mod tests {
                     let data_addr = vm_layout_constants::HEAP_START;
 
                     let metadata_1_spec = SideMetadataSpec {
+                        name: "metadata_1_spec",
                         is_global: true,
                         offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
                         log_num_of_bits: 4,
-                        log_min_obj_size: 6,
+                        log_bytes_in_region: 6,
                     };
 
                     let metadata_2_spec = SideMetadataSpec {
+                        name: "metadata_2_spec",
                         is_global: true,
                         offset: SideMetadataOffset::layout_after(&metadata_1_spec),
                         log_num_of_bits: 3,
-                        log_min_obj_size: 7,
+                        log_bytes_in_region: 7,
                     };
 
                     let metadata = SideMetadataContext {
@@ -377,10 +387,11 @@ mod tests {
                         + (vm_layout_constants::BYTES_IN_CHUNK << 1);
 
                     let metadata_1_spec = SideMetadataSpec {
+                        name: "metadata_1_spec",
                         is_global: true,
                         offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
                         log_num_of_bits: 1,
-                        log_min_obj_size: constants::LOG_BYTES_IN_WORD as usize,
+                        log_bytes_in_region: constants::LOG_BYTES_IN_WORD as usize,
                     };
 
                     let metadata = SideMetadataContext {
@@ -431,32 +442,36 @@ mod tests {
 
                     #[cfg(target_pointer_width = "64")]
                     let metadata_1_spec = SideMetadataSpec {
+                        name: "metadata_1_spec",
                         is_global: false,
                         offset: SideMetadataOffset::addr(LOCAL_SIDE_METADATA_BASE_ADDRESS),
                         log_num_of_bits: 4,
-                        log_min_obj_size: 9,
+                        log_bytes_in_region: 9,
                     };
                     #[cfg(target_pointer_width = "64")]
                     let metadata_2_spec = SideMetadataSpec {
+                        name: "metadata_2_spec",
                         is_global: false,
                         offset: SideMetadataOffset::layout_after(&metadata_1_spec),
                         log_num_of_bits: 3,
-                        log_min_obj_size: 7,
+                        log_bytes_in_region: 7,
                     };
 
                     #[cfg(target_pointer_width = "32")]
                     let metadata_1_spec = SideMetadataSpec {
+                        name: "metadata_1_spec",
                         is_global: false,
                         offset: SideMetadataOffset::rel(0),
                         log_num_of_bits: 4,
-                        log_min_obj_size: 9,
+                        log_bytes_in_region: 9,
                     };
                     #[cfg(target_pointer_width = "32")]
                     let metadata_2_spec = SideMetadataSpec {
+                        name: "metadata_2_spec",
                         is_global: false,
                         offset: SideMetadataOffset::layout_after(&metadata_1_spec),
                         log_num_of_bits: 3,
-                        log_min_obj_size: 7,
+                        log_bytes_in_region: 7,
                     };
 
                     let metadata = SideMetadataContext {
