@@ -331,7 +331,11 @@ options! {
     // Should finalization be disabled?
     no_finalizer:          bool                 [env_var: true, command_line: true]  [always_valid] = false,
     // Should reference type processing be disabled?
-    no_reference_types:    bool                 [env_var: true, command_line: true]  [always_valid] = false,
+    // If reference type processing is disabled, no weak reference processing work is scheduled,
+    // and we expect a binding to treat weak references as strong references.
+    // We disable weak reference processing by default, as we are still working on it. This will be changed to `false`
+    // once weak reference processing is implemented properly.
+    no_reference_types:    bool                 [env_var: true, command_line: true]  [always_valid] = true,
     // The zeroing approach to use for new object allocations. Affects each plan differently. (not supported)
     nursery_zeroing:       NurseryZeroingOptions[env_var: true, command_line: true]  [always_valid] = NurseryZeroingOptions::Temporal,
     // How frequent (every X bytes) should we do a stress GC?
